/*
 * Generic access to direct memory access functionality.
 *
 * Copyright (C) 2024 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <string.h>
#include "dma-generic.h"

/* Function declarations reproduced here to avoid clock identifier conflicts in
   the chip-specific headers. */

extern Dma_chip *jz4730_dma_chip(l4_addr_t, l4_addr_t, Cpm_chip *);
//extern Dma_chip *jz4740_dma_chip(l4_addr_t, l4_addr_t, Cpm_chip *);
extern Dma_chip *jz4780_dma_chip(l4_addr_t, l4_addr_t, Cpm_chip *);
extern Dma_chip *x1600_dma_chip(l4_addr_t, l4_addr_t, Cpm_chip *);

struct dma_function
{
  const char *name;
  Dma_chip *(*function)(l4_addr_t, l4_addr_t, Cpm_chip *);
};

static struct dma_function functions[] = {
  {"jz4730", jz4730_dma_chip},
  //{"jz4740", jz4740_dma_chip},
  {"jz4780", jz4780_dma_chip},
  {"x1600", x1600_dma_chip},
  {NULL, NULL}
};

Dma_chip *new_dma_chip(const char *name, l4_addr_t start, l4_addr_t end, Cpm_chip *cpm)
{
  for (struct dma_function *f = functions; f->name != NULL; f++)
  {
    if (!strcmp(name, f->name))
      return f->function(start, end, cpm);
  }

  return NULL;
}
